/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shoppingcart.webflow;

import examples.e2e.b2c.util.B2CPortalConstants;
import com.bea.portal.appflow.PortalAppflowConstants;

import javax.servlet.http.HttpServletRequest;

import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.LinkedList;
import java.util.Iterator;

import com.beasys.commerce.util.Logger;
import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.foundation.exception.InvalidArgumentException;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartLine;

import com.bea.p13n.appflow.exception.ProcessingException;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

/**
 * UpdateShoppingCartQuantitiesIP reads in the new quantity string for
 * each item in the shopping cart from the
 * <code>HttpServletRequest</code> object.  Each quantity string is
 * parsed, and if it is a positive integer, the shopping cart line for
 * that item is updated with the new quantity.  Invalid quantity
 * values are ignored.  The
 * <code>PipelineSessionConstants.SHOPPING_CART</code> pipeline
 * session attribute is updated with a new <code>ShoppingCart</code>
 * object that reflects the changed quantities.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 * @see com.beasys.commerce.webflow.CommerceInputProcessor
 */

public class UpdateShoppingCartQuantitiesIP extends InputProcessorSupport
{
    /**
     * See class documentation above.
     *
     * @param request <code>HttpServletRequest</code> object for the
     * current request
     * @return String containing "success" if the operation was
     * successful
     * @throws ProcessingException If the
     * <code>HttpRequestConstants.SHOPPING_CART</code> parameter is
     * not available in the request
     */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {
        LinkedList itemsToRemove = null;
	
        // Get pipeline session from http session
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART,
								       namespace, pipelineSession);

        if(shoppingCart == null) {
            throw new ProcessingException("ebusiness-shoppingcart:null.shoppingcart:" +
					  PipelineSessionConstants.SHOPPING_CART);
        }
	
        // Create a number formatter object
        NumberFormat quantityFormat = NumberFormat.getInstance();
        quantityFormat.setMinimumFractionDigits(0);
        quantityFormat.setMaximumFractionDigits(0);
        quantityFormat.setParseIntegerOnly(true);

        ProductItem[] updatedItems = new ProductItem[shoppingCart.getShoppingCartLineCollection().size()];
        double[] deltas = new double[shoppingCart.getShoppingCartLineCollection().size()];
        int i = 0;

        // loop through all the shopping cart lines and set the new quantities
        Iterator iterator = shoppingCart.getShoppingCartLineCollection().iterator();
        while(iterator.hasNext()) {
            ShoppingCartLine shoppingCartLine = (ShoppingCartLine)iterator.next();
            ProductItem productItem = shoppingCartLine.getProductItem();
            updatedItems[i] = productItem;

            String sku = productItem.getKey().getIdentifier();
            String quantityString = request.getParameter("NewQuantity_"+sku);  // Olivier: Really the trim line below is the right one when the NullPtrExc is found
            String itemName = productItem.getName();

            quantityString = request.getParameter("NewQuantity_"+sku).trim();

            // Parse the string using the number formatter
            ParsePosition parsePosition = new ParsePosition(0);
            Number number = quantityFormat.parse(quantityString, parsePosition);

            double currentQuantity = 0;
             
            try {
               currentQuantity = shoppingCart.findShoppingCartLine(sku).getQuantity();
            } 
	    catch (Exception ignore) {
            }

            // Make sure the number was valid
            if(number != null) {
                // Check to make sure the whole string was parsed
                if(parsePosition.getIndex() == quantityString.length()) {
                    double newQuantity = (double)number.doubleValue();
		    
                    // If quantity equals zero, add to the list of items to remove
                    if(newQuantity == 0) {
                        if(itemsToRemove == null) {
                            itemsToRemove = new java.util.LinkedList();
                        }
                        itemsToRemove.add(sku);
                        deltas[i] = -currentQuantity;
                    }
                    else {
                        // Only quantities greater that 0 or less
                        // than/equal to Integer.MAX_VALUE are valid
                        if(newQuantity > 0 && newQuantity <= Integer.MAX_VALUE) {
                            // If we got through all the tests, update the quantity
                            try {
                                deltas[i] = newQuantity - currentQuantity;
                                shoppingCart.setProductItemQuantity(productItem, newQuantity);
                            }
                            catch(InvalidArgumentException iae) {
                                Logger.getInstance().log(Logger.LOG_DEBUG, 
							 "Exception occurred in UpdateShoppingCartQuanatitiesIP.process()", iae);
                            }
                        }
                    }
                }
            }
            i++;
        }

        // Remove all items that had a zero quantity
        if(itemsToRemove != null) {
            iterator = itemsToRemove.iterator();
            while(iterator.hasNext()) {
                String sku = (String)iterator.next();

                try {
                    shoppingCart.removeProductItem(sku);
                }
                catch(InvalidArgumentException iae)
                {
                    Logger.getInstance().log(Logger.LOG_DEBUG, "Exception occurred in UpdateShoppingCartQuanatitiesIP.process()", iae);
                }
            }
        }

        setSessionAttribute(PipelineSessionConstants.UPDATED_QUANTITY_DELTAS, deltas, namespace, pipelineSession );   
        setSessionAttribute(PipelineSessionConstants.UPDATED_PRODUCT_ITEMS, updatedItems, namespace, pipelineSession );   

        // Operation succeeded
        // Get the portlet webflow event from the HTTP request 
        String event = (String) request.getParameter( PortalAppflowConstants.PORTLET_WEBFLOW_EVENT_PARAMETER );

        // Depending on the type of event, we store a specific doc portlet location attribute
        setSessionAttribute( B2CPortalConstants.DOC_PORTLET_LOCATION, "step1." + event, namespace, pipelineSession );

        return "success";
    }
}




