/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2c.order.pipeline;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Set;
import java.util.ArrayList;
import java.util.Iterator;
import javax.ejb.Handle;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.beasys.commerce.axiom.units.Price;
import com.beasys.commerce.axiom.units.Money;
import com.beasys.commerce.axiom.units.PriceHome;
import com.beasys.commerce.axiom.contact.Address;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartLine;
import com.beasys.commerce.ebusiness.customer.Customer;
import com.beasys.commerce.ebusiness.customer.CustomerPk;
import com.beasys.commerce.ebusiness.customer.CustomerHome;
import com.beasys.commerce.ebusiness.shipping.ShippingMethodValue;
import com.beasys.commerce.ebusiness.order.Order;
import com.beasys.commerce.ebusiness.order.OrderPk;
import com.beasys.commerce.ebusiness.order.OrderHome;
import com.beasys.commerce.ebusiness.order.OrderValue;
import com.beasys.commerce.ebusiness.order.Order;
import com.beasys.commerce.ebusiness.order.OrderManager;
import com.beasys.commerce.ebusiness.order.OrderManagerHome;
import com.beasys.commerce.ebusiness.order.OrderLine;
import com.beasys.commerce.ebusiness.order.OrderLineHome;

import com.beasys.commerce.axiom.util.weblogic.helper.WebLogicHelper;
import java.sql.*;

import com.bea.commerce.ebusiness.price.quote.Quote;
import com.bea.commerce.ebusiness.discount.association.DiscountAssociationMgr;
import com.bea.commerce.ebusiness.discount.association.DiscountAssociationMgrHome;

import com.beasys.commerce.webflow.PipelineSessionConstants;
import examples.e2e.b2c.webflow.CommercePipelineComponentSupport;
import examples.e2e.b2c.util.B2CPortalConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;

import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;
import com.bea.p13n.util.jdbc.Sequencer;
import com.bea.p13n.util.jdbc.SequencerFactory;

/**
 * This pipeline component is responsible for persisting the order.
 * It expects the following artifacts in the PipelineSession:
 *
 * <ul>
 * <li>ShoppingCart</li>
 * <li>User name</li>
 * <li>ShippingAddress</li>
 * <li>ShippingMethod</li>
 * <li>SplittingPreference</li>
 * </ul>
 *
 * After validating that these artifacts exist in the PipelineSession,
 * it creates an Order entity. Subsequent
 * PipelineComponent objects can access the Order
 * entity using the Order Handle from the PipelineSession.
 */
public class CommitOrderPC extends CommercePipelineComponentSupport
{
    private static final String ORDER_MANAGER = ".BEA_commerce.OrderManager";
    private static final String CUSTOMER_HOME = ".BEA_commerce.Customer";
    private static final String DISCOUNT_ASS_MGR = ".BEA_commerce.DiscountAssociationMgr";

    /**
     * See class documentation above.
     *
     * @param pipelineSession the <code>PipelineSession</code> objects that the component should use
     * @return The modified PipelineSession object
     * @throws com.beasys.commerce.foundation.exception.PipelineFatalException if any of the expected
     *          attributes are not present in the pipeline session, or if the shopping cart was empty
     */
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) 
	throws PipelineException
    {
        OrderPk orderPk = new OrderPk();
        OrderManager orderManager = null;
        OrderValue orderValue = null;
        Handle orderHandle = null;
        Customer customer = null;

	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the shopping cart from the pipeline session
	ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART, 
								       namespace, pipelineSession);
        if(shoppingCart == null) {
            throw new PipelineException("webflow-targetprocessor:invalid.session");
        }

        // Get special instructions from the pipeline session
	String specialInstructions = (String) getSessionAttribute(PipelineSessionConstants.SPECIAL_INSTRUCTIONS, 
								  namespace, pipelineSession);
        if(specialInstructions == null) {
            throw new PipelineException("ebusiness-order:order.missing.specialinstructions.exception:" + 
					PipelineSessionConstants.SPECIAL_INSTRUCTIONS);
        }

        // Get splitting preference from the pipeline session
	String splittingPreference = (String) getSessionAttribute(PipelineSessionConstants.SPLITTING_PREFERENCE, 
								  namespace, pipelineSession);
        if(splittingPreference == null)
        {
            throw new PipelineException("ebusiness-order:order.missing.splittingpreference.exception:" + 
					PipelineSessionConstants.SPLITTING_PREFERENCE);
        }

        // Get shipping method from the pipeline session
	ShippingMethodValue shippingMethodValue = (ShippingMethodValue) getSessionAttribute(PipelineSessionConstants.SHIPPING_METHOD, 
											    namespace,
											    pipelineSession);
        if(shippingMethodValue == null) {
            throw new PipelineException("ebusiness-order:order.missing.shippingmethod.exception:" + 
					PipelineSessionConstants.SHIPPING_METHOD);
        }

        // Set the shipping address
	Address shippingAddress = (Address) getSessionAttribute(PipelineSessionConstants.SHIPPING_ADDRESS, 
								namespace, pipelineSession);
        if(shippingAddress == null) {
            throw new PipelineException("ebusiness-order:order.missing.shippingaddress.exception:" + 
					PipelineSessionConstants.SHIPPING_ADDRESS);
        }

        // Make sure the shopping cart is not empty
        if(shoppingCart.shoppingCartLineCollectionSize() <= 0) {
            throw new PipelineException("ebusiness-order:order.empty.shoppingCart.exception");
        }

        // Set the adjustments
	Quote adjustments = (Quote) getSessionAttribute(PipelineSessionConstants.ORDER_ADJUSTMENTS, namespace,
							pipelineSession);
        if(adjustments == null) {
            throw new PipelineException("ebusiness-order:order.empty.adjustments.exception");
        }

        // Set the discount sets
	Set userDiscountIds   = (Set) getSessionAttribute(PipelineSessionConstants.DISCOUNT_IDS, namespace,
							  pipelineSession);
	Set globalDiscountIds = (Set) getSessionAttribute(PipelineSessionConstants.GLOBAL_DISCOUNT_IDS, 
							  namespace, pipelineSession);

        // Get the customer
        CustomerPk customerPk = null;
        String customerRef = ApplicationHelper.getApplicationName() +
          CUSTOMER_HOME;
        try {
            CustomerHome customerHome = (CustomerHome) JndiHelper.lookupNarrow(customerRef,
									       CustomerHome.class);
	    String userId = (String) getSessionAttribute(PipelineSessionConstants.USER_NAME, namespace,
							 pipelineSession);

            if(userId == null) {
                throw new PipelineException("ebusiness-order:order.missing.username.exception:" + 
					    PipelineSessionConstants.USER_NAME);
            }
            customerPk = new CustomerPk(userId);
            customer = customerHome.findByPrimaryKey(customerPk);
        }
        catch(NamingException te) {
            throw new PipelineException("ebusiness-order:customer.lookup.exception:" + customerRef, te);
        }
        catch(FinderException fe) {
            throw new PipelineException("ebusiness-order:customer.finder.exception:" + customerPk.identifier, fe);
        }
        catch(RemoteException re) {
            throw new PipelineException("ebusiness-order:get.customer.remote.exception", re);
        }

        // Create a primary key for the order
        Sequencer sequencer = SequencerFactory.createSequencer("ORDER_SEQUENCE", getDataSource());
        Long longSequence;
        try {
            longSequence = new Long(sequencer.getNext());
        }
        catch(SQLException se) {
            throw new PipelineException("ebusiness-order:order.sequencer.exception", se);
        }
        orderPk.identifier = longSequence.toString();

        // Get the order manager
        String orderMgrRef = ApplicationHelper.getApplicationName() +
          ORDER_MANAGER;
        try
        {
            OrderManagerHome oh = (OrderManagerHome) JndiHelper.lookupNarrow(orderMgrRef,
									     OrderManagerHome.class);
            orderManager = oh.create();
        }
        catch(Exception e) {
            throw new PipelineException("ebusiness-order:order.lookup.exception:" + orderMgrRef, e);
        }

	// create the order value object and call set by value
        try {
	    // Creates the order value
            orderValue = new OrderValue();

	    // set the identifier, mostly for the commitorder.jsp
	    orderValue.identifier = orderPk.identifier;

	    // Set the price and sub-total
            orderValue.price    = shoppingCart.getTotal();
            orderValue.subtotal = shoppingCart.getSubtotal().getValue();

            // set the shipping method
            orderValue.shippingMethod = shippingMethodValue.identifier;

            // set the shipping address
            orderValue.shippingAddress = shippingAddress;

            // set the special instructions
            orderValue.specialInstructions = specialInstructions;

            // set the splitting preference
            orderValue.splittingPreference = splittingPreference;

            // set the date of the order
            orderValue.createdDate = new java.sql.Date( new java.util.Date().getTime() );

            // set the list of order lines
            orderValue.orderLines = new ArrayList();

            // Get the order line id sequencer
            sequencer = SequencerFactory.createSequencer("ORDER_LINE_ID_SEQUENCE", getDataSource());

            // iterate through the shopping cart, and populate the Order with the contents in the shopping cart.
            Iterator cartLinesIterator = shoppingCart.shoppingCartLineCollectionIterator();
            while(cartLinesIterator.hasNext()) {
                ShoppingCartLine cartLine = (ShoppingCartLine) cartLinesIterator.next();

                OrderLine orderLine = (OrderLine) OrderLineHome.create();
                orderLine.setProductIdentifier(cartLine.getProductItem().getKey().getIdentifier());
                orderLine.setQuantity(cartLine.getQuantity());
                orderLine.setUnitPrice(cartLine.getUnitPrice());
                orderLine.setDescription(cartLine.getProductItem().getName());
                orderLine.setMsrp(cartLine.getProductItem().getMsrp());

                orderLine.setTotalLineAmount(cartLine.getDiscountedPrice().getValue() );

                // Set the sequence number outside of the ejbStore tx
                // type in OrderLineCRUD.
                try {
                    orderLine.setId(sequencer.getNext());
                }
                catch(SQLException se) {
                    throw new PipelineException("ebusiness-order:order.sequencer.exception", se);
                }

                Money tax = cartLine.getTax();
                if(tax != null) {
                    orderLine.setTax(tax);
                }

                orderLine.setShipping(cartLine.getShipping());
                orderValue.orderLines.add(orderLine);
            }

            // Set the shipping code on the order
            orderValue.shipping = shoppingCart.getShipping();

            // Set the customer on the order.
            orderValue.customer = customer;

            // Look up order home
            orderHandle = orderManager.createOrder(orderPk, orderValue, adjustments);
        }
        catch(RemoteException re) {
            throw new PipelineException("ebusiness-order:order.commit.remote.exception", re);
        }

        // Get the discount association manager and update the uses and global uses
        String discountAssMgrRef = ApplicationHelper.getApplicationName() +
          DISCOUNT_ASS_MGR;
        try {
	    DiscountAssociationMgrHome admh = (DiscountAssociationMgrHome) JndiHelper.lookupNarrow(discountAssMgrRef,
												   DiscountAssociationMgrHome.class);
	    DiscountAssociationMgr dam = admh.create();
	    
            if(userDiscountIds != null) {
                dam.updateUses(customerPk, userDiscountIds);
            }
            if (globalDiscountIds != null) {
                dam.updateUsesGlobal(customerPk, globalDiscountIds);
            }
            
            // Get the discount presentations for this order and store it in the order value
            orderManager.queryOrderAdjusments(orderPk, orderValue);
        }
        catch(Exception e) {
            throw new PipelineException("ebusiness-order:order.updatediscountuses.exception" + discountAssMgrRef, e);
        }

	
        // Store the order handle object in the pipeline session
	setRequestAttribute(PipelineSessionConstants.ORDER_HANDLE, orderHandle, namespace, 
			    requestContext, pipelineSession);

        // Set the order value object in the pipeline (use request scope because we only need it on the next screen)
	setRequestAttribute(PipelineSessionConstants.ORDER_VALUE, orderValue, namespace, 
			    requestContext, pipelineSession);

        // Set the order shipping method value object in the pipeline
        // (use request scope because we only need it on the next
        // screen)
	setRequestAttribute(PipelineSessionConstants.ORDER_SHIPPING_METHOD, shippingMethodValue, 
			    namespace, requestContext, pipelineSession);

       	// Indicate that order history cache is now stale and needs to be updated
        setSessionAttribute(B2CPortalConstants.ORDER_HISTORY_LIST_VALID, new Boolean(false), 
			    namespace, pipelineSession);


        // The payment is really the total of the order, not of
        // the shopping cart!
        Price authorizationAmount = PriceHome.create();
        try {
            authorizationAmount.setValue(orderValue.price.getValue());
            authorizationAmount.setCurrency(orderValue.price.getCurrency());
        }
        catch(com.beasys.commerce.foundation.exception.InvalidArgumentException e) {
            throw new PipelineException("ebusiness-order:invalid.argument", e);
        }


        // Return modified pipeline session
        return pipelineSession;
    }
}
