/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/payment/CreditCardService.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.payment;

import java.rmi.RemoteException;
import javax.ejb.FinderException;

import com.beasys.commerce.foundation.Session;
import com.beasys.commerce.axiom.contact.CreditCard;
import com.beasys.commerce.axiom.units.Price;
import com.beasys.commerce.ebusiness.payment.TransactionResponse;
import com.beasys.commerce.ebusiness.payment.AuthorizationException;
import com.beasys.commerce.ebusiness.payment.InvalidCreditCardException;
import com.beasys.commerce.ebusiness.payment.GeneralServiceException;
import com.beasys.commerce.ebusiness.payment.ServiceUnavailableException;
import com.beasys.commerce.ebusiness.payment.ServiceConfigurationException;
import com.beasys.commerce.ebusiness.payment.CaptureException;
import com.beasys.commerce.ebusiness.payment.PaymentException;
import com.beasys.commerce.ebusiness.payment.SettleException;

/**
 * A <code>CreditCardService</code> is a Stateless Session EJB that provides services related
 * to the authorization, capture, and settlement of credit card transactions. 
 * <HTML>
 * <BR>
 * It is designed as a service provider interface behind which integrations with various payment
 * solutions are implemented.  The business methods implement a very standard workflow that is
 * associated with the completion of credit card transactions.  The current state of the 
 * transaction is maintained and each action is journaled.  
 * <BR>
 * <UL>
 * <LI>
 * Each transaction is initiated with a request to authorize.  This authorization generally results
 * in the creation of a persistent {@link PaymentTransaction}.
 * The state of the payment and the key for that PaymentTransaction is returned in a
 * {@link TransactionResponse} as well as service specific information.
 * A handle for that PaymentTransaction can be obtained from the
 * TransactionResponse. 
 * </LI>
 * <LI>
 * In the event that the initial authorization fails due to a failure to connect to the
 * payment authorization service, it is is possible to retry the authorization using the 
 * reauthorize method.
 * </LI>
 * <LI>
 * An authorized transaction can then be captured or settled depending on the way that
 * the service is configured.
 * </LI>
 * <LI>
 * Alternatively it is possible to complete the entire transaction in a single authorizeAndCapture.
 * </LI>
 * </UL>
 * <B>
 * Note that these methods may behave differently based on the payment model used by
 * a merchant.  Refer to the product documentation for guidance on how to configure 
 * WLCS to work with your business model. The documentation included here makes reference
 * to concepts that are more fully explained in the documentation.
 * </B>
 * </HTML>
 *
 * @see com.beasys.commerce.ebusiness.payment.CreditCardServiceHome
 * @see com.beasys.commerce.ebusiness.payment.TransactionResponse
 * @see com.beasys.commerce.ebusiness.payment.PaymentTransaction
 */
public interface CreditCardService extends Session
{
    /**
     * Reserve credit on the supplied card for the amount specified.  
     * The amount of the transaction is deducted from the "open to buy" in the customer's credit balance, 
     * but the funds are not actually transferred to the merchant. Funds are transferred from the customer's 
     * credit balance to the merchants account only after settling. 
     * <br><br>
     * <b>This method should only be used for terminal-based payment *
     * models</b>.  Merchants who are using a terminal-based processor
     * must * perform a capture and settlement procedure in order for
     * the funds * from the sale to be transferred to their account.
     * This is * accomplished by a subsequent call to capture and/or
     * settle * (depending on the Auto Mark/Auto Settle processor
     * configuration).  * <br><br> It is also possible to void a
     * transaction that has not yet * been captured or settled using
     * the voidTransaction method.
     *
     * @param card CreditCard against which the amount will be authorized.
     * @param amount the amount of credit to authorize.
     * @param useAvs boolean that tells the service if the billing address on the card is to be verified.
     * @return a {@link TransactionResponse} that refers to the resulting PaymentTransaction. 
     * @throws InvalidCreditCardException If the associated credit card is invalid
     * @throws AuthorizationException If the associated credit card could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization service is unavailable
     * @throws ServiceConfigurationException If the authorization service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse authorize(CreditCard card, 
					 Price amount, 
					 boolean useAvs) 
	throws RemoteException , AuthorizationException, 
	InvalidCreditCardException, ServiceUnavailableException, 
	ServiceConfigurationException, PaymentException, GeneralServiceException;
    
    /**
     * Reserve credit on the supplied card for the amount specified.
     * The amount of the transaction is deducted from the "open to
     * buy" in the customer's credit balance, but the funds are not
     * actually transferred to the merchant. Funds are transferred
     * from the customer's credit balance to the merchants account
     * only after settling.  <br><br> <b>This method should only be
     * used for host-based payment models</b>.  Merchants who are
     * using a host-based post-authorization capture processor must
     * perform a capture and settlement procedure in order for the
     * funds from the sale to be transferred to their account.  This
     * is accomplished by a subsequent call to capture and/or settle.
     * However, in the case of a host-based authorization-capture
     * processor, the capture and settlement of the transaction are
     * done by the processor at the time of authorization.  <br><br>
     * It is also possible to void a transaction that has not yet been
     * captured or settled using the voidTransaction method.
     *
     * @param card CreditCard against which the amount will be
     * authorized.
     * @param amount the amount of credit to authorize.
     * @param useAvs boolean that tells the service if the billing
     * address on the card is to be verified.
     * @return a {@link TransactionResponse} that refers to the
     * resulting PaymentTransaction.
     * @throws InvalidCreditCardException If the associated credit
     * card is invalid
     * @throws AuthorizationException If the associated credit card
     * could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization
     * service is unavailable
     * @throws ServiceConfigurationException If the authorization
     * service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse authorizeAndCapture(CreditCard card, 
						   Price amount, 
						   boolean useAvs) 
	throws RemoteException, AuthorizationException, InvalidCreditCardException, 
	ServiceUnavailableException, ServiceConfigurationException, PaymentException, 
        GeneralServiceException;

    /**
     * Atempt to authorize a payment transaction that is in the retry
     * state.  The payment transaction is updated with the current
     * date and a transaction entry is added to the payment
     * transaction as the result of the authorization attempt. The
     * modified payment transaction and any service-specific results
     * are then returned.  <br><br> <b>This method should only be used
     * for terminal-based payment models</b>.
     *
     * @param paymentTransactionId The ID of the payment transaction
     * to reauthorize
     * @param useAvs Flag indicating whether to use address
     * verfication
     * @return a TransactionResponse that refers to the resulting
     * PaymentTransaction.
     * @throws FinderException If the payment transaction with the
     * specified ID could not be found
     * @throws InvalidCreditCardException If the associated credit
     * card is invalid
     * @throws AuthorizationException If the associated credit card
     * could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization
     * service is unavailable
     * @throws ServiceConfigurationException If the authorization
     * service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse reauthorize(String paymentTransactionId,
					   boolean useAvs)
	throws FinderException, InvalidCreditCardException, AuthorizationException, 
	ServiceUnavailableException, ServiceConfigurationException, 
	PaymentException, GeneralServiceException, RemoteException;

    /**
     * Atempt to authorize a payment transaction that is in the retry
     * state.  The payment transaction is updated with the current
     * date and a transaction entry is added to the payment
     * transaction as the result of the authorization attempt. The
     * modified payment transaction and any service-specific results
     * are then returned.  <br><br> <b>This is method should only be
     * used for host-based payment models</b>.
     *
     * @param paymentTransactionId The ID of the payment transaction
     * to reauthorize
     * @param useAvs Flag indicating whether to use address
     * verfication
     * @return a TransactionResponse that refers to the updated
     * PaymentTransaction.
     * @throws FinderException If the payment transaction with the
     * specified ID could not be found
     * @throws InvalidCreditCardException If the associated credit
     * card is invalid
     * @throws AuthorizationException If the associated credit card
     * could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization
     * service is unavailable
     * @throws ServiceConfigurationException If the authorization
     * service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse reauthorizeAndCapture(String paymentTransactionId,
						     boolean useAvs)
	throws FinderException, InvalidCreditCardException, AuthorizationException,
	ServiceUnavailableException, ServiceConfigurationException, PaymentException,
        GeneralServiceException, RemoteException;

    /**
     * Marks a portion of a payment transaction for settlement such
     * that it will be processed in the next batch.  The amount can be
     * less than or equal to the authorization amount.  <br><br>
     * Before a transaction can be settled (and funds transfered into
     * a merchant's account) it must first be captured.  This marks
     * the transaction such that it will be settled as part of a batch
     * process.  <b>This method should be used for terminal-based
     * payment models with a Manual Mark processor configuration</b>.
     * <b>This method should be used for host-based, post
     * authorization and capture payment model</b>.
     * @param paymentTransactionId The ID of the payment transaction
     * to reauthorize.
     * @param amount The amount of the transaction to capture.
     * @return A TransactionResponse that refers to the updated
     * PaymentTransaction.
     * @throws CaptureException If it was not possible to capture the
     * amount from the transaction.
     * @throws ServiceUnavailableException If the underlying service
     * is unreachable.
     * @throws PaymentException On general payment exception.
     * @throws RemoteException On remote error
     */
    public TransactionResponse capture(String transactionId, Price amount) 
	throws RemoteException, CaptureException, ServiceUnavailableException, PaymentException;

    /**
     * Query the Payment server for transactions which match the
     * supplied parameters.  The caller need only supply non-null
     * values for those parameters that they want to query against,
     * but must supply at least one non-null parameter.  <br><br> <b>
     * This method is implemented as a pass through to the underlying
     * service provider.  As such all return information is service
     * specific.  Refer to the service provider manual for details on
     * return codes and results.  </b>
     * @param transactionId The ID of a specific transaction. 
     * @throws RemoteException On remote error
     */
    public TransactionResponse queryTransactions(String transactionId, 
						 String transactionType,
						 String transactionStatus, 
						 String transactionOrigin,
						 String cardType,
						 java.sql.Date startTime,
						 java.sql.Date endTime,
						 Price lowAmount, 
						 Price highAmount,
						 Integer maxNumberOfTransactions) 
        throws RemoteException, PaymentException;

    /**
     * Abort a transaction which was previously submitted.  The following transactions may be voided:
     * <UL>
     * <LI>
     * HOST_AUTH_CAPTURE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * HOST_POST_AUTH_CAPTURE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * AUTO_MARK_AUTO_SETTLE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * MANUAL_MARK_AUTO_SETTLE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * AUTO_MARK_MANUAL_SETTLE transactions in the captured state.
     * </LI>
     * <LI>
     * MANUAL_MARK_MANUAL_SETTLE transactions in the captured state.
     * </LI>
     * </UL>
     *
     * @param transactionId The ID of the payment transaction to void.
     * @throws PaymentException General payment error.
     * @throws RemoteException On remote error.
     */
    public TransactionResponse voidTransaction(String transactionId) throws RemoteException, PaymentException;

    /**
     * Finalize a transaction by transferring a portion of the funds
     * previously captured from the customer's account to the
     * merchant's account.  The amount can be less than or equal to
     * the captured amount.  <br><br> <b>This method should be used
     * for terminal-based payment models with a Manual Settle
     * processor configuration</b>.
     * @param transactionId The ID of the payment transaction to
     * settle.
     * @param amount The amount of the transaction to mark for
     * settlement.
     * @throws SettleException If the transaction cannot be settled.
     * @throws ServiceUnavailableException If the authorization service is unavailable
     * @throws PaymentException General payment error.
     * @throws RemoteException On remote error
     */
    public TransactionResponse settle(String transactionId, Price amount) throws RemoteException,
	SettleException, ServiceUnavailableException, PaymentException;

    /**
     * The batch-query message is used to update and reconcile the
     * status of a transaction committed in a given batch.  This
     * message is used to determine if a particular transaction has
     * failed, and is essential for payment processors where the
     * status of an item cannot be determined correctly from the
     * output fields of a batch-commit message.  <br><br> <b> This
     * method is implemented as a pass through to the underlying
     * service provider.  As such all return information is service
     * specific.  Refer to the service provider manual for details on
     * return codes and results.  <\b>
     * @param transactionId The ID of the payment transaction to query status for.
     * @throws RemoteException On remote error.
     */
    public TransactionResponse batchQuery(String transactionId) throws RemoteException, PaymentException;
}


