/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/payment/pipeline/PaymentAuthorizationTerminalPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */
package examples.e2e.b2c.payment.pipeline;

import java.rmi.RemoteException;
import java.util.*;
import javax.ejb.*;

import com.beasys.commerce.ebusiness.order.Order;
import com.beasys.commerce.ebusiness.payment.*;
import com.beasys.commerce.axiom.contact.CreditCard;
import com.beasys.commerce.axiom.units.Price;
import com.beasys.commerce.util.Logger;

import com.bea.p13n.exceptions.SystemException;
import com.bea.p13n.appflow.exception.PipelineException;

import examples.e2e.b2c.payment.CreditCardService;

/**
 * This <code>PipelineComponent</code> authorizes a credit card for a
 * purchase of a given amount using a terminal-based payment model.
 * The terminal model allows the flexibilty to explicitly settle a
 * given payment transaction at the end of each day or shift.  This
 * differs from the host model which is used by {@link
 * PaymentAuthorizationHostPC}.  This class is a concrete extension of
 * the <code>PaymentAuthorizationPC</code> abstract base class.
 * <br><br> For a description of the various supported payment models,
 * refer to the WLCS payment documentation.
 *
 * @see com.beasys.commerce.ebusiness.payment.pipeline.PaymentAuthorizationHostPC
 * @see com.beasys.commerce.ebusiness.payment.pipeline.PaymentAuthorizationPC
 * @see com.beasys.commerce.foundation.pipeline.PipelineComponent
 */
public class PaymentAuthorizationTerminalPC extends PaymentAuthorizationPC
{
    /**
     * Authorize a credit card for a purchase of a given amount.
     *
     * @param service The credit card service with which to authorize
     * @param creditCard The credit card to authorize
     * @param amount The amount of the purchase
     * @throws PipelineNonFatalException if the credit card service is unavailable or incorrectly configured.
     * @throws AuthorizationFailureException if the credit card specified is invalid.
     * @throws AuthorizationRejectedException if the credit card specified fails authorization.
     * @throws PipelineFatalException on general service error.
     * @throws RemoteException on remote error.
     */
    public TransactionResponse authorize(CreditCardService service,
                                         CreditCard creditCard,
                                         Price amount,
                                         Order order) throws PipelineException, RemoteException
    {
        // Authorize the credit card using the Terminal method
        try {
            return service.authorize(creditCard, amount, true);
        }
        // The authorization service is unavailable
        catch (ServiceUnavailableException e)
        {
            if(DEBUG)
            {
                Logger.getInstance().debug("In PaymentAuthorizationTerminalPC: detected system unavailable exception - throwing non fatal Pipeline exception.");

            }

            Handle paymentTransactionHandle = e.getResponse().getPaymentTransactionHandle();
            order.setPaymentTransaction((PaymentTransaction)paymentTransactionHandle.getEJBObject());

            throw new PipelineException("ebusiness-payment:credit.card.authorization.error", e);
        }
        // The credit card is invalid
        catch(InvalidCreditCardException e) {
            throw new AuthorizationFailureException("ebusiness-payment", "credit.card.authorization.failed", e);
        }
        // The credit card could not be authorized
        catch(AuthorizationException e) {
            throw new AuthorizationRejectedException("ebusiness-payment", "credit.card.authorization.rejected", e);
        }
        // Something's wrong with the service configuration
        catch(ServiceConfigurationException ce) {
            throw new PipelineException("ebusiness-payment:credit.card.authorization.error", ce);
        }
        // General service error
        catch(GeneralServiceException gse) {
            throw new PipelineException("ebusiness-payment:credit.card.authorization.error", gse);
        }
        catch (PaymentException pe) {
            throw new PipelineException(pe.getMessage(), pe);
        }
        // System-level error
        catch(SystemException e) {
            throw new PipelineException("ebusiness-payment:credit.card.authorization.error", e);
        }
    }
}
