/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shipping/pipeline/CalculateShippingPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

package examples.e2e.b2c.shipping.pipeline;

import java.util.LinkedList;

import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shipping.ShippingMethodValue;
import com.beasys.commerce.axiom.units.Money;
import com.beasys.commerce.axiom.units.MoneyHome;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartLine;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;

/**
 * This calculates a fixed shipping cost. It uses the price attribute
 * on the selected shipping method to calculate the shipping cost.
 */
public class CalculateShippingPC extends PipelineComponentSupport
{
    private static final int CHEAP_GROUND_RATE = 4;
    private static final int EXPENSIVE_GROUND_RATE = 6;
    private static final int AIR_RATE = 10;

    /**
     * Calculates the shipping cost and updates the shopping cart to
     * reflect the change.
     * @param PipelineSession the PipelineSession.
     * @throws <code>InvalidPipelineSessionStateException</code> if the
     * <code>ShoppingCart</code> is missing in the <code>PipelineSession</code>.
     * @throws <code>PipelineFatalException</code> in the following cases:
     *<UL>
     *<LI>
     *    If the shopping cart is empty.
     *</LI>
     *<LI>
     *    If the shiping method is not specified.
     *</LI>
     *<LI>
     *    If a currency is invalid.
     *</LI>
     *</UL>
     */

    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
        int cost;

	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART, 
								       namespace, pipelineSession);
        if(shoppingCart == null) {
            throw new PipelineException("webflow-targetprocessor:invalid.session");
        }

        // Check that the shopping cart isn't empty
        if(shoppingCart.isEmpty() == true) {
            throw new PipelineException("ebusiness-shipping:shoppingcart.is.empty");
        }

        // Reset the shoppingCart shipping cost if it ever was set (CR044915)
        shoppingCart.setShipping(MoneyHome.create());

        // Get the shipping method from the pipeline session
        ShippingMethodValue shippingMethod = (ShippingMethodValue) getSessionAttribute(PipelineSessionConstants.SHIPPING_METHOD, 
										       namespace, pipelineSession);
        if(shippingMethod == null) {
            throw new PipelineException("ebusiness-shipping:shippingmethod.not.in.pipeline:" + 
					PipelineSessionConstants.SHIPPING_METHOD);
        }


        // Split the total shipping across the number of items.
        // Some tax may be calculated based on the per-item shipping costs.
        // This is only an approximation of shipping costs per line item as
        // it doesn't take into account weight.
        int number_of_items = 0;
        LinkedList shoppingCartLineList = (LinkedList)shoppingCart.getShoppingCartLineCollection();

        for(int i=0 ; i < shoppingCartLineList.size(); i++) {
            ShoppingCartLine shoppingCartLine = (ShoppingCartLine)shoppingCartLineList.get(i);
            number_of_items += (int) shoppingCartLine.getQuantity();
        }
	
        // Get the price for this shipping method
        double totalShipping = shippingMethod.price.getValue();
        double shippingCostLeft = totalShipping;

        // Iterate through all the lines except the last one and split up the shipping costs between them
        for(int i=0 ; i < shoppingCartLineList.size() - 1; i++) {
            ShoppingCartLine shoppingCartLine = (ShoppingCartLine)shoppingCartLineList.get(i);

            // Calculate the line shipping
            double lineShipping = (totalShipping * shoppingCartLine.getQuantity()) / number_of_items;

            // Subtract the shipping for this line from the total shipping
            shippingCostLeft -= lineShipping;

            // Set the shipping cost on the shopping cart line
            Money lineShippingMoney = MoneyHome.create();
            lineShippingMoney.setValue(lineShipping);
            try {
                lineShippingMoney.setCurrency(shippingMethod.price.getCurrency());
            }
            catch (com.beasys.commerce.foundation.exception.InvalidArgumentException e) {
                throw new PipelineException("ebusiness-shipping:invalid.parameter.currency");
            }
            shoppingCartLine.setShipping(lineShippingMoney);
        }

        // Set the shipping costs for the last to be what ever is
        // left.  we do this to prevent any rounding errors in total
        // shipping
        ShoppingCartLine shoppingCartLine = (ShoppingCartLine) shoppingCartLineList.get(shoppingCartLineList.size() 
											- 1);
        Money lineShippingMoney = MoneyHome.create();
        lineShippingMoney.setValue(shippingCostLeft);
        try {
            lineShippingMoney.setCurrency(shippingMethod.price.getCurrency());
        }
        catch (com.beasys.commerce.foundation.exception.InvalidArgumentException e) {
            throw new PipelineException("ebusiness-shipping:invalid.parameter.currency");
        }
        shoppingCartLine.setShipping(lineShippingMoney);

        // Update the shopping cart
        shoppingCart.setShoppingCartLineCollection( shoppingCartLineList );

        // Update the pipeline session
        setSessionAttribute(PipelineSessionConstants.SHOPPING_CART, shoppingCart, namespace, pipelineSession);

        return pipelineSession;
    }
}


