/**
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.   All Rights Reserved
 *
 */
package examples.e2e.b2c.tax.pipeline;

import javax.ejb.CreateException;
import java.rmi.RemoteException;
import javax.naming.NamingException;

import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;

import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

import examples.e2e.b2c.tax.TaxCalculator;
import examples.e2e.b2c.tax.TaxCalculatorHome;
import examples.wlcs.sampleapp.tax.TaxWebServiceHelper;

/**
 * The <code>TaxPC</code> is the base class for all tax pipeline
 * components, it defines helper methods that acquire a handle to the
 * Tax Calculator service and verify that the required attributes are
 * available prior to the execution of business logic.
 */
public abstract class TaxPC extends PipelineComponentSupport
{
    /**
     * TaxPC is the abstract base class for tax pipeline components
     * The noargs constructor does nothing.
     */
    public TaxPC()
    {
    }

    /**
     * Get the name of the TaxCalculator Session EJB.  It does this
     * based on the property "tax.calculator.jndi.name" in the
     * weblogiccommerce.properties file
     * @return the JNDI name of the TaxCalculator to be
     * used by the tax pipeline components.
     */
    protected String getTaxCalculatorJndiName()
        throws PipelineException
    {
        String jndiName = TaxWebServiceHelper.getTaxServiceClientMBean().getTaxCalculatorJNDIName();

        if(jndiName == null || jndiName.length() == 0)
            throwFatal("tax.pipeline.jndi.missing");

        return jndiName;
    }

    /**
     * Return a <code>TaxCalculator</code> instance.
     *
     * @return the TaxCalculator instance to be used by the tax
     * pipeline components.
     *
     */
    protected TaxCalculator getTaxCalculator()
        throws PipelineException
    {
        try {
	    String tcHome = ApplicationHelper.getApplicationName() + "." + getTaxCalculatorJndiName();
            TaxCalculatorHome taxCalculatorHome = (TaxCalculatorHome) JndiHelper.lookupNarrow(tcHome,
											      TaxCalculatorHome.class);
            return taxCalculatorHome.create();
        }
        catch(RemoteException re) {
            throwFatal("tax.pipeline.remote.exception", ApplicationHelper.getApplicationName() + "." + getTaxCalculatorJndiName());
        }
        catch(CreateException ce) {
            throwFatal("tax.pipeline.ejbcreate.exception", ApplicationHelper.getApplicationName() + "." + getTaxCalculatorJndiName());
        }
        catch(NamingException ne) {
            throwFatal("tax.pipeline.lookup.exception", ApplicationHelper.getApplicationName() + "." + getTaxCalculatorJndiName());
        }

        return null;
    }

    /**
     *
     * Return the message bundle name (for logging purposes).
     *
     * @return the name of the message bundle used when reporting
     * errors.
     *
     */
    protected String getMessageBundleName()
    {
        return "ebusiness-tax";
    }

    /**
     * Throws a PipelineFatalException exception
     *
     * @param message the identifier of the message in the message bundle
     * @param param the parameter for a parameterized message
     *
     */
    protected void throwFatal(String message, String param)
        throws PipelineException
    {
        throw new PipelineException (getMessageBundleName() + ":" + message + ":" + param);
    }

    /**
     * Throws a PipelineFatalException exception
     *
     * @param message the identifier of the message in the message bundle
     *
     */
    protected void throwFatal(String message)
        throws PipelineException
    {
        throw new PipelineException (getMessageBundleName() + ":" + message);
    }

    /**
     * Throws a PipelineFatalException exception
     *
     * @param message the identifier of the message in the message bundle
     * @param e the exception that should be embedded
     *
     */
    protected void throwFatal(String message, Exception e)
        throws PipelineException
    {
        throw new PipelineException (getMessageBundleName() + ":" + message, e);
    }

    /**
     * Checks that a required attribute is present in the pipeline
     * session.
     * @param pipelineSession the PipelineSession object
     * @param attrib the name of an attribute
     * @return true is the attribute is defined in the pipeline session
     */
    protected boolean isDefined(PipelineSession pipelineSession, String attrib) throws PipelineException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();
        return(getSessionAttribute(attrib, namespace, pipelineSession) != null);
    }

    /**
     * Checks that a required attribute is present in the pipeline
     * session. If the parameter is present it is returned, otherwise
     * an exception is thrown.
     *
     * @param pipelineSession the PipelineSession object
     * @param attrib the name of an attribute
     * @return the value of the attribute
     */
    protected Object requireDefined(PipelineSession pipelineSession, String attrib)
        throws PipelineException
    {
        if(isDefined(pipelineSession, attrib) == false)
            throwFatal("tax.pipeline.attributenotavailable.exception", attrib);

		String namespace = (String) pipelineSession.getDefaultNamespace();
        return getSessionAttribute(attrib, namespace, pipelineSession);
    }
}










