/**
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.   All Rights Reserved
 *
 */
package examples.e2e.b2c.tax.pipeline;

import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ArrayList;
import java.util.List;
import javax.ejb.*;

import com.beasys.commerce.ebusiness.tax.TaxLine;
import com.beasys.commerce.ebusiness.tax.TaxLineHome;
import com.beasys.commerce.ebusiness.tax.TaxLineList;
import com.beasys.commerce.ebusiness.tax.TaxLineListHome;
import com.beasys.commerce.ebusiness.tax.TaxParameters;
import com.beasys.commerce.ebusiness.tax.TaxParametersHome;
import com.beasys.commerce.ebusiness.tax.TaxResult;
import com.beasys.commerce.ebusiness.tax.TaxInformation;

import com.beasys.commerce.foundation.exception.InvalidArgumentException;
import com.beasys.commerce.axiom.util.helper.CommercePropertiesHelper;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartLine;
import com.beasys.commerce.axiom.units.Money;
import com.beasys.commerce.axiom.units.Price;
import com.beasys.commerce.axiom.units.PriceHome;
import com.beasys.commerce.axiom.units.Quantity;
import com.beasys.commerce.axiom.units.QuantityHome;
import com.beasys.commerce.axiom.contact.Address;
import com.beasys.commerce.axiom.contact.AddressHome;
import com.beasys.commerce.ebusiness.catalog.ProductItem;

import com.bea.commerce.ebusiness.price.quote.Adjustment;
import com.bea.commerce.ebusiness.price.quote.AdjustmentDetail;
import com.bea.commerce.ebusiness.price.quote.AdjustmentType;
import com.bea.commerce.ebusiness.price.quote.OrderAdjustment;

import com.bea.p13n.management.ApplicationHelper;
import javax.management.InstanceNotFoundException;

import com.bea.p13n.management.MBeanAccess;
import com.bea.p13n.management.ServiceConfigurationMBean;
import com.bea.commerce.mbeans.TaxServiceClientMBean;


/**
 * The <code>TaxPipeLineService</code> class provides helper methods
 * to load the tax parameters and update the shopping cart's tax
 * information.
 */
public class TaxPipelineService
{
    
    private static boolean debug = false;

    
    /**
     * Returns the TaxParameters for the shipping address.
     *
     * @param commit whether the tax parameters should include the
     * commit option.
     * @return a populated TaxParameters object created based on the
     * shipping address and the merchants parameters.
     */
    public TaxParameters getTaxParameters(Address shippingAddress, boolean commit)
    {
        return getTaxParameters(shippingAddress, commit, false);
    }

    /**
     * Returns the TaxParameters for the shipping address.
     *
     * @param commit whether the tax parameters should include * the
     * commit option.
     * @param returnTransaction true if the transaction is a return and not a credit
     * @return a populated TaxParameters object created based on the *
     * shipping address and the merchants parameters.
     */
    public TaxParameters getTaxParameters(Address shippingAddress, boolean commit, boolean returnTransaction)
    {
        TaxParameters taxParameters = TaxParametersHome.create();

        // Set all locations
        taxParameters.putLocation(TaxParameters.SHIPTO, shippingAddress);
        taxParameters.putLocation(TaxParameters.SHIPFROM, TaxPipelineService.getTaxAddressFromMBean(true));
        taxParameters.putLocation(TaxParameters.ORDERORIGIN,  TaxPipelineService.getTaxAddressFromMBean(false));

       	taxParameters.addAttrib(TaxParameters.CURRENCY, TaxPipelineService.getTaxServiceClientMBean().getCurrency());

        taxParameters.addAttrib(TaxParameters.COMMIT,new Boolean(commit));

        //Type of transaction it is
        if(returnTransaction == true) {
            taxParameters.addAttrib(TaxParameters.CREDIT, new Boolean(false));
	}

	//
	// Point of title passage
	//
	if(TaxPipelineService.getTaxServiceClientMBean().getTitlePassageLocation() != null) {
	    taxParameters.addAttrib(TaxParameters.TITLEPASSAGE,
				     TaxPipelineService.getTaxServiceClientMBean().getTitlePassageLocation());
	}
	
	if(TaxPipelineService.getTaxServiceClientMBean().getCompanyID() != null) {
	    taxParameters.addAttrib(TaxParameters.COMPANYID,
				     TaxPipelineService.getTaxServiceClientMBean().getCompanyID());
	}
	
	if(TaxPipelineService.getTaxServiceClientMBean().getBusinessLocationCode() != null) {
	    taxParameters.addAttrib(TaxParameters.BUSINESSLOCATIONCODE,
				     TaxPipelineService.getTaxServiceClientMBean().getBusinessLocationCode());
	}
	
	//
	// type of tax
	//
	if(TaxPipelineService.getTaxServiceClientMBean().getTaxType() != null) {
	    taxParameters.addAttrib(TaxParameters.TAXTYPE,
				     TaxPipelineService.getTaxServiceClientMBean().getTaxType());
	}
	
	if(TaxPipelineService.getTaxServiceClientMBean().getSellerRegistrationNumber() != null) {
	    taxParameters.addAttrib(TaxParameters.SELLERREGISTRATIONNUMBER,
				     TaxPipelineService.getTaxServiceClientMBean().getSellerRegistrationNumber());
	}
	
	if(TaxPipelineService.getTaxServiceClientMBean().getTaxDebug() != null) {
	    taxParameters.addAttrib(TaxParameters.TAXDEBUG,
				     TaxPipelineService.getTaxServiceClientMBean().getTaxDebug());
	}
	
        return taxParameters;

    }


    /**
     * Returns a TaxLineList from the ShoppingCartLines in the ShoppingCart
     *
     * @param shoppingCart the user's shopping cart
     * @return the tax calculated per-item for the shopping cart
     *
     */
    public TaxLineList getTaxLineList(ShoppingCart shoppingCart)
    {
         // derive the total amount of the order level discounts
         // by aggregating the order level discounts
         boolean doesExistOrderLevelDiscount = false;
         double orderLevelDiscountTotal = 0;
         double orderLevelDiscountRemainder = 0;
         double cartSubTotalBeforeOrderLevelDiscounts = shoppingCart.getSubtotal().getValue();
         
         ArrayList orderdiscounts = shoppingCart.getOrderDiscountPresentations();
         if (orderdiscounts != null)
         {
            Iterator orderdisIter = orderdiscounts.iterator();
            while (orderdisIter.hasNext())
            {
               Adjustment curAd = (Adjustment)orderdisIter.next();
               orderLevelDiscountTotal += curAd.getAdjustmentAmount().getValue();
            }
            orderLevelDiscountRemainder = orderLevelDiscountTotal;
         }
         if (orderLevelDiscountTotal != 0) 
         {
            doesExistOrderLevelDiscount = true;
         }
         if (debug)
         {
            System.out.println("TaxPipelineService: orderLevelDiscountTotal = "+orderLevelDiscountTotal);               
            System.out.println("TaxPipelineService: cartSubTotalBeforeOrderLevelDiscounts = "+cartSubTotalBeforeOrderLevelDiscounts);               
         }         

         // compute the original total for the shopping cart before the order
         // level discounts were applied
         // This value is used when proportionally distributing the order level discounts
         
        TaxLineList  taxLineList = TaxLineListHome.create( );
        Iterator it = shoppingCart.shoppingCartLineCollectionIterator( );
        TaxLine taxLine = null;
        LinkedList list = new LinkedList( );
        String shippingCurrency = null;
      
        // iterate through the shopping cart lines and create a corresponding tax line
        while ( it.hasNext( ) )
        {
            taxLine = TaxLineHome.create( );
            ShoppingCartLine shoppingCartLine = (ShoppingCartLine) it.next( );

            Money originalUnitPrice = shoppingCartLine.getUnitPrice( );
            Money discountedUnitPrice = shoppingCartLine.getDiscountedPrice( );
            discountedUnitPrice.setValue(discountedUnitPrice.getValue() / shoppingCartLine.getQuantity( ));

            if (debug)
            {
               System.out.println("TaxPipelineService: originalUnitPrice = "+originalUnitPrice.getValue());               
               System.out.println("TaxPipelineService: discountedUnitPrice = "+discountedUnitPrice.getValue());               
            }
            
            Price originalLineTotal = PriceHome.create( );
            Price discountLineTotal = PriceHome.create( );
            Price discountAmount = PriceHome.create( );

            try
            {
               // set Price in the tax line
               // note: the price on the tax line must be the TOTAL price
               // i.e. unit price * quantity.
               originalLineTotal.setCurrency( originalUnitPrice.getCurrency( ) );
               originalLineTotal.setValue( originalUnitPrice.getValue( ) * shoppingCartLine.getQuantity( ) );
               taxLine.setPrice( originalLineTotal );

               // save off the currency for the shipping line
               shippingCurrency = originalUnitPrice.getCurrency();
               
               // set Discount Amount in the tax line
               {
                  // 1. setup the discounted price for the line
                  discountLineTotal.setCurrency( discountedUnitPrice.getCurrency( ) );
                  discountLineTotal.setValue( discountedUnitPrice.getValue( ) * shoppingCartLine.getQuantity( ) );
                  if (debug)
                  {
                     System.out.println("TaxPipelineService: step 1 discountLineTotal = "+discountLineTotal.getValue());               
                  }

                  // 2. compute the amount of the order level discount to share for this line
                  double lineOrderLevelDiscountShare = 0;
                  double lineTotalDiscount = 0;
                  if (doesExistOrderLevelDiscount)
                  {
                     // 2a. compute the share of the order level discount that
                     // should be assigned to this order line for tax purposes
                     // subtract from a total so that rounding errors are avoided
                     // if this is the last shopping cart line, use the remainder value
                     if (it.hasNext())
                     {
                        lineOrderLevelDiscountShare = (discountLineTotal.getValue() / 
                           cartSubTotalBeforeOrderLevelDiscounts)*orderLevelDiscountTotal;
                        orderLevelDiscountRemainder = orderLevelDiscountRemainder-
                           lineOrderLevelDiscountShare;
                     }
                     else
                     {
                        lineOrderLevelDiscountShare = orderLevelDiscountRemainder;
                     }
                     if (debug)
                     {
                        System.out.println("TaxPipelineService: step 2a lineOrderLevelDiscountShare = "+lineOrderLevelDiscountShare);               
                     }
                     
                     // 2b. do not allow a negative taxable amount for a given line
                     if ((discountLineTotal.getValue() - lineOrderLevelDiscountShare) < 0)
                     {
                        lineOrderLevelDiscountShare = discountLineTotal.getValue();
                     }
                     if (debug)
                     {
                        System.out.println("TaxPipelineService: step 2b lineOrderLevelDiscountShare = "+lineOrderLevelDiscountShare);               
                     }
                     
                     // 2c. compute the final discounted total for this shopping
                     // cart line. This is the total line price with the line
                     // level and order level discounts factored in
                     discountLineTotal.setValue( discountLineTotal.getValue() - 
                        lineOrderLevelDiscountShare );
                     if (debug)
                     {
                        System.out.println("TaxPipelineService: step 2c discountLineTotal = "+discountLineTotal.getValue());               
                     }
                  }

                  // 3. compute the actual discount amount for this line
                  // (original price - discounted line price)
                  discountAmount.setCurrency( originalUnitPrice.getCurrency( ) );
                  discountAmount.setValue(originalLineTotal.getValue() -
                     discountLineTotal.getValue());
                  if (debug)
                  {
                     System.out.println("TaxPipelineService: step 3 discountAmount = "+discountAmount.getValue());               
                  }
                  
                  // 4. set the discounted amount to the tax line
                  taxLine.setDiscountedPriceAmount( discountAmount );
               }
               
               // set the tax code
               taxLine.setTaxCode( shoppingCartLine.getProductItem().getTaxCode() );

               // set the quantity
               Quantity quantity = QuantityHome.create( );
               quantity.setCount( shoppingCartLine.getQuantity( ) );
               taxLine.setQuantity( quantity );

               // CR056298 10-11-01
               // removed the per line shippingTotal charge since that method
               // cannot handle shipping discounts. Add shipping as a 
               // seperate tax line below.

               if (debug)
               {
                  System.out.println( "************* TaxPipelineService: Tax Line ***********" );
                  System.out.println( "************* Original Line Total: " + originalLineTotal.getValue() );
                  System.out.println( "************* Original Line Currency: " + originalLineTotal.getCurrency() );
                  System.out.println( "************* Discount Line Total: " + discountLineTotal.getValue() );
                  System.out.println( "************* Discount Line Currency: " + discountLineTotal.getCurrency() );
                  System.out.println( "************* Discount Amount Total: " + discountAmount.getValue() );
                  System.out.println( "************* Discount Amount Currency: " + discountAmount.getCurrency() );
                  System.out.println( "************* Tax Code: " + shoppingCartLine.getProductItem().getTaxCode() );
                  System.out.println( "************* Quantity: " + shoppingCartLine.getQuantity( ) );
                  System.out.println( "************* ShippingLineAmount: " + shoppingCartLine.getShipping().getValue() );
                  System.out.println( "**********************************" );
               }
               
                list.add( taxLine );
            }
            catch( InvalidArgumentException e )
            {
               e.printStackTrace();
            }
        }
         
      
         // CR056298 10-11-01
         // add shipping as an additional tax line
         try
         {
            // setup the shipping prices
            Price originalShippingPrice = PriceHome.create( );
            originalShippingPrice.setCurrency(shippingCurrency);

            Price discountShippingAmount = PriceHome.create( );
            discountShippingAmount.setCurrency(shippingCurrency);
            
            // get the total shipping information by looking at the adjustments
            // generally there will be just one adjustment, containing one or two 
            // adjustment details.
            double discountAmt = 0;
            ArrayList discounts = shoppingCart.getShippingDiscountPresentations();
            Iterator disIter = discounts.iterator();
            while (disIter.hasNext())
            {
               Adjustment curAd = (Adjustment)disIter.next();

               List details = curAd.getDetails();
               Iterator detailIter = details.iterator();
               
               // iterate through the shipping adjustment details
               // there will always be one detail to provide the shipping amount (I_SHIPPING_
               // there is an optional second detail providing the discount (I_SHIPPING_DISCOUNT)
               while (detailIter.hasNext())
               {
                  AdjustmentDetail curDetail = (AdjustmentDetail)detailIter.next();
                  if (debug)
                  {
                     System.out.println("TaxPipelineService: detail: "+curDetail.toString());               
                  }
                  
                  if (curDetail.getType().getInt() == AdjustmentType.I_SHIPPING_BASE_PRICE)
                  {
                     // obtain the shipping amount
                     // the amount is priceaftershippingadded-initialprice
                     originalShippingPrice.setValue(curDetail.getEndPrice().getValue()-
                        curDetail.getInitialPrice().getValue());
                  }
                  else if (curDetail.getType().getInt() == AdjustmentType.I_SHIPPING_DISCOUNT)
                  {
                     // add to the discount amount
                     // the amount of the discount is initialprice - priceafterdiscount
                     discountAmt = discountAmt+
                        (curDetail.getInitialPrice().getValue()-curDetail.getEndPrice().getValue()); 
                  }
               }
            }

            // setup the discount amount for shipping
            // note: this is the amount of the discount 
            // if no discounts were present this value will be zero
            // i.e. (originalShippingPrice - discountShippingAmount = discountShippingPrice)
            if ((originalShippingPrice.getValue()-discountAmt) < 0) 
            {
               discountAmt = originalShippingPrice.getValue(); 
            }
            discountShippingAmount.setValue(discountAmt);

            if (debug)
            {
               System.out.println("TaxPipelineService: original shipping price: "+originalShippingPrice.getValue());               
               System.out.println("TaxPipelineService: discount shipping amount: "+discountShippingAmount.getValue());               
            }
            
            // create a new tax line
            taxLine = TaxLineHome.create( );
            
            // set the prices and tax code
            // CONFIGURE: the general shipping tax code for Taxware is 40000
            //             you may need to change this, consult your tax dept.
            taxLine.setTaxCode( ""+40000 );
            taxLine.setPrice( originalShippingPrice );
            taxLine.setDiscountedPriceAmount( discountShippingAmount );

            if (debug)
            {
               System.out.println( "************* TaxPipelineService: Shipping Tax Line ***********" );
               System.out.println( "************* Original Line Total: " + originalShippingPrice.getValue() );
               System.out.println( "************* Original Line Currency: " + originalShippingPrice.getCurrency() );
               System.out.println( "************* Discount Amount Total: " + discountShippingAmount.getValue() );
               System.out.println( "************* Discount Amount Currency: " + discountShippingAmount.getCurrency() );
               System.out.println( "************* Tax Code: " + taxLine.getTaxCode() );
               System.out.println( "**********************************" );
            }
            
            // add the tax line to the set
            list.add( taxLine );
         }
         catch( InvalidArgumentException e )
         {
            e.printStackTrace();
         }

        taxLineList.setTaxLineList( list );

        return taxLineList;
    }

    /**
     * Writes the TaxResult information into the ShoppingCart
     *
     * @param taxResult the calculated tax for the shopping cart
     * @param shoppingCart the user's shopping cart
     * @return the updated shopping cart that includes the tax
     *
     */
    public ShoppingCart updateShoppingCart(TaxResult taxResult, ShoppingCart shoppingCart)
    {
        LinkedList taxInfoList = (LinkedList) taxResult.getLine( );
        LinkedList shoppingCartLineList = (LinkedList) shoppingCart.getShoppingCartLineCollection( );
        ShoppingCartLine shoppingCartLine = null;
        TaxInformation taxInformation = null;
      
        // iterate through the taxInfo lines. note that the last line is the artificial
        // shipping line
        int i = 0;
        for ( i=0 ; i < taxInfoList.size( )-1; i++ )
        {
            shoppingCartLine = (ShoppingCartLine) shoppingCartLineList.get( i );
            taxInformation = (TaxInformation) taxInfoList.get( i );
            shoppingCartLine.setTax( taxInformation.getTotal() );

            if (debug)
            {
               System.out.println("TaxPipelineService: Tax for line "+i+" = "+taxInformation.getTotal());
            }
            shoppingCartLineList.set( i, shoppingCartLine );
        }

        // do something with the shipping tax, add it to the last line
         Money shippingTax = shoppingCartLine.getTax();
         taxInformation = (TaxInformation) taxInfoList.get( i );
         shippingTax.setValue(shippingTax.getValue() + taxInformation.getTotal().getValue());
         if (debug)
         {
            System.out.println("TaxPipelineService: Shipping tax = "+taxInformation.getTotal());
         }
         shoppingCartLine.setTax( shippingTax );
         shoppingCartLineList.set( i-1, shoppingCartLine );
      
        // save the updated shoppingCartLineList into the shopping cart
        shoppingCart.setShoppingCartLineCollection( shoppingCartLineList );
        return shoppingCart;
    }

    private static TaxServiceClientMBean getTaxServiceClientMBean()
    {
	TaxServiceClientMBean taxMBean = null;
	
	if(taxMBean == null) {
	    try	{
		taxMBean = (TaxServiceClientMBean)ApplicationHelper.getServiceConfigurationMBean("TaxServiceClient");
	    }
	    catch(Exception e) {
		System.err.println("caught exception getting tax mbean - ");
		e.printStackTrace();
	    }
	}
	
	return(taxMBean);
    }
    
    /**
     * generates an Address from the individual elements in the mbean
     */
    private static Address getTaxAddressFromMBean(boolean isShipFrom)
    {
	Address	address = AddressHome.create();
	
	if(isShipFrom) {
	    address.setState(TaxPipelineService.getTaxServiceClientMBean().getShipFromProvince());
	    address.setCity(TaxPipelineService.getTaxServiceClientMBean().getShipFromCity());
	    address.setPostalCode(TaxPipelineService.getTaxServiceClientMBean().getShipFromPostalCode());
	    address.setGeoCode(TaxPipelineService.getTaxServiceClientMBean().getShipFromLocationCode());
	    address.setCountry(TaxPipelineService.getTaxServiceClientMBean().getShipFromCountryCode());
	}
	else {
	    address.setState(TaxPipelineService.getTaxServiceClientMBean().getOrderOriginProvince());
	    address.setCity(TaxPipelineService.getTaxServiceClientMBean().getOrderOriginCity());
	    address.setPostalCode(TaxPipelineService.getTaxServiceClientMBean().getOrderOriginPostalCode());
	    address.setGeoCode(TaxPipelineService.getTaxServiceClientMBean().getOrderOriginLocationCode());
	    address.setCountry(TaxPipelineService.getTaxServiceClientMBean().getOrderOriginCountryCode());
	}
	
	return(address);
    }
}
