
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/BuyerFactory.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.e2e.common;

import com.bea.p13n.util.debug.Debug;
import com.bea.commerce.ebusiness.price.quote.Money;

import org.xml.sax.InputSource;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;

import java.util.Calendar;
import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Properties;

import weblogic.utils.classloaders.GenericClassLoader;


public class BuyerFactory {

    private static Debug debug =
            Debug.getInstance( BuyerFactory.class );

    /**
     * Used to format and parse date strings in XML documents.
     * Dates should be constructed in the form "yyyyMMdd"
     * (see java.text.SimpleDateFormat for details) to be successfully
     * parsed.
     */
    public static final String xmlDateFormatString =
            "yyyyMMdd";

    /**
     * The base file name for XML documents containing static purchase
     * orders.  Files are located using the value of the system property
     * defined by the constant DATA_LOCATION_PROPERTY as a path.  
     *
     * @see #DATA_LOCATION_PROPERTY
     * @see #getStaticPOSources
     */ 
    public static final String STATIC_PO_BASE_NAME = "staticPO";

    /**
     * The name of the file produced by the workflow engine containing a
     * collection of quotes in response to a query for price and availability (QPA).
     * The file is located using the value of the system property
     * defined by the constant DATA_LOCATION_PROPERTY as a path.
     *
     * @see #DATA_LOCATION_PROPERTY
     */
    public static final String QUOTE_FILE = "E2E_AggregatedQPAResponse.xml";

    /**
     * The name of the file produced by the workflow engine containing a
     * purchase order. The file is located using the value of the system property
     * defined by the constant DATA_LOCATION_PROPERTY as a path.
     *
     * @see #DATA_LOCATION_PROPERTY
     */
    public static final String PO_FILE = "E2E_PO.xml";

    /**
     * The name of the file produced by the workflow engine containing
     * purchase order data. The file is located using the value of the system property
     * defined by the constant DATA_LOCATION_PROPERTY as a path.
     *
     * @see #DATA_LOCATION_PROPERTY
     */
    public static final String PO_DATA_FILE = "E2E_PO.data";

    /**
     * The name of the file produced by the workflow engine containing
     * an acknowledged purchase order. The file is located using the value
     * of the system property defined by the constant DATA_LOCATION_PROPERTY
     * as a path.
     *
     * @see #DATA_LOCATION_PROPERTY
     */
    public static final String PO_ACK_FILE = "E2E_POAcknowledgement.xml";

    /**
     * The name of the file produced by the workflow engine containing
     * acknowledged purchase order data. The file is located using the value
     * of the system property defined by the constant DATA_LOCATION_PROPERTY
     * as a path.
     *
     * @see #DATA_LOCATION_PROPERTY
     */
    public static final String PO_ACK_DATA_FILE = "E2E_POAcknowledgement.data";

    private static final String XML_SUFFIX = ".xml";

    /**
     * If this property is set this class will use canned
     * <code>PurchaseQuote</code> and <code>PO</code> objects
     * instead of creating the objects from parsed XML data files.
     *
     * @see PurchaseQuote
     * @see PO
     * @see #QUOTE_FILE
     * @see #PO_FILE
     * @see #PO_ACK_FILE
     */
    private static final String TEST_PROP =
            "samples.e2e.common.buyerFactory.useTestImpl";
    private static boolean useTestImpl;

    static {
        try {
            useTestImpl = Boolean.getBoolean( TEST_PROP );

        }
        catch ( Exception e ) {
            debug.out( "unable to get system property( " +
                    TEST_PROP + " )" );
        }
    }

    /**
     * The value of this system property is used to determine the location
     * of XML files containing application data.  In most cases, these files are
     * generated by the workflow engine in response to a query or submission.
     *
     * @see #STATIC_PO_BASE_NAME
     * @see #QUOTE_FILE
     * @see #PO_FILE
     * @see #PO_ACK_FILE
     * @see #getQuoteSource
     * @see #getPOSource
     * @see #getStaticPOSources
     */
    public static final String DATA_LOCATION_PROPERTY = "e2eSample.b2b.path.data";

    private static String dataLocation;


    private BuyerFactory() {
    } 


    /**
     * Returns a <code>QPA</code> object populated from the method parameters.
     *
     * @param productId the identifier for the product to be quoted.
     * @param requestId the unique identifier for this QPA request.
     * @param description the product description.
     * @param desiredUnitPrice the price the buyer would like to pay for the product.
     * @param desiredQty the quantity of product the buyer would like quoted.
     * @param creationDate the creation date for this QPA.
     * @param desiredDate the date the purchaser desires to receive the product.
     *
     * @return a <code>QPA</code> object.
     *
     * @see QPA
     */
    public static QPA createQPA( String productId,
            String requestId,
            String description,
            Money desiredUnitPrice,
            long desiredQty,
            Calendar creationDate,
            Calendar desiredDate) {

        return new QPAImpl( productId, requestId, description,
                desiredUnitPrice, desiredQty,
                creationDate, desiredDate );
    }

    /**
     * Returns a <code>List</code> of <code>PurchaseQuote</code> objects
     * populated from the data supplied in the XML document specified as
     * an <code>org.xml.sax.InputSource</code>.
     *
     * @param <code>org.xml.sax.InputSource</code> XML to be parsed.
     * @return a <code>List</code> of <code>PurchaseQuote</code> objects.
     * @throws ParsingException if the XML cannot be parsed successfully.
     *
     * @see PurchaseQuote
     */
    public static List parseQuote( InputSource source ) throws ParsingException {

        List rtnVal = null;

        if ( useTestImpl ) {

            rtnVal = TestBuyerFactory.createQuote( source );
        }
        else {
            if ( source == null ) {

                rtnVal = null;
            }
            else {

                QuoteParser parser = new QuoteParser( source );
                rtnVal = parser.parse();
            }
        }

        return rtnVal;
    }

    /**
     * Returns a <code>List</code> of <code>PO</code> objects
     * populated from the data supplied in the XML document.
     *
     * @param xml <code>InputSource</code> to use to create <code>PurchaseQuote</code>s.
     */
    public static PO parsePO( InputSource source ) throws ParsingException {

        PO rtnVal = null;

        if ( useTestImpl ) {

            return(PO) TestBuyerFactory.createPO( source ).get( 0 );
        }
        else {

            if ( source == null ) {

                rtnVal = null;
            }
            else {

                POParser parser = new POParser( source );
                rtnVal = parser.parse();
            }
        }

        return rtnVal;
    }

    /**
     * Returns a <code>List</code> of <code>PO</code> objects
     * populated from the data supplied in the XML document.
     *
     * @param xml <code>InputSource</code> to use to create <code>PurchaseQuote</code>s.
     */
    public static List parsePOs( List srcs ) throws ParsingException {

        if ( useTestImpl ) {

            return TestBuyerFactory.createPO( null );
        }

        List list = new ArrayList( srcs.size() );
        Iterator it = srcs.iterator();
        InputSource src = null;
        PO po = null;

        while ( it.hasNext() ) {

            src = (InputSource) it.next();

            if ( src == null ) {

                continue;
            }

            po = parsePO( src );
            if ( po == null ) {

                continue;
            }

            list.add( po );
        }

        if ( debug.ON ) {
            debug.out( "list.size()(" + list.size() + ")" );
        }

        return list;
    }


    public static InputSource getQuoteSource() {

        ClassLoader cl = Thread.currentThread().getContextClassLoader();

        return getSource( cl, QUOTE_FILE );
    }

    public static InputSource getPOSource() {

        ClassLoader cl = Thread.currentThread().getContextClassLoader();

        InputSource src = getSource( cl, PO_ACK_FILE );

        if ( src == null ) {

            // if that file doesn't exist maybe this one will  ... but maybe not
            src = getSource( cl, PO_FILE );
        }

        // src still may be null
        return src;
    }

    public static List getStaticPOSources() {

        ClassLoader cl = Thread.currentThread().getContextClassLoader();

        List list = new LinkedList();

        boolean done = false;
        InputSource src = null;
        int cnt = 0;
        String resourceName = null;

        while ( ! done ) {

            cnt++;
            resourceName = STATIC_PO_BASE_NAME + cnt + XML_SUFFIX;

            if ( debug.ON ) {
                debug.out( "resourceName(" + resourceName + ")" );
            }

            src = getSource( cl, resourceName );

            if ( debug.ON ) {
                debug.out( "src(" + src + ")" );
            }

            if ( src == null ) {

                done = true;
                break;
            }

            list.add( src );
        }

        if ( debug.ON ) {
            debug.out( "list.size()(" + list.size() + ")" );
        }

        return list;
    }

    public static InputSource getSource( ClassLoader cl, String name ) {

        InputSource src = null;
        InputStream is = cl.getResourceAsStream( name );

        if ( debug.ON ) {
            debug.out( "is(" + is + ")" );
        }

        if ( is == null ) {

            src = null;
        }
        else {

            src = new InputSource( new BufferedInputStream( is ) );
        }

        return src;
    }

    public static void deleteXML() throws NullPointerException,
            SecurityException, RuntimeException {

        getDataLocation();

        deleteFile( dataLocation, QUOTE_FILE );

        deletePoXML();

        return;
    }

    public static void deletePoXML() throws NullPointerException,
            SecurityException, RuntimeException {

        getDataLocation();

        deleteFile( dataLocation, PO_FILE );
        deleteFile( dataLocation, PO_ACK_FILE );
        deleteFile( dataLocation, PO_DATA_FILE );
        deleteFile( dataLocation, PO_ACK_DATA_FILE );

        return;
    }

    public static boolean doesQuoteXMLExist() {

        return doesFileExist( dataLocation, QUOTE_FILE );
    }

    public static boolean doesPOAckXMLExist() {

        return doesFileExist( dataLocation, PO_ACK_FILE );
    }

    public static String getDataLocation() throws NullPointerException,
            SecurityException, RuntimeException {

        if ( dataLocation == null ) {

            try {

                dataLocation = getProperty( DATA_LOCATION_PROPERTY );
            }
            catch ( RuntimeException ex ) {

                // LOG
                debug.out(
                        "unable to delete files: cannot determine data " +
                        "location, exception( " + ex + " )", ex );

                throw ex;
            }
        }

        if ( dataLocation == null ) {

            // LOG
            debug.out(
                    "the path to the XML data may not be null, " +
                    "set the system property( " + DATA_LOCATION_PROPERTY + 
                    " ) appropriately." );
            throw new NullPointerException(
                    "the path to the XML data may not be null, " +
                    "set the system property( " + DATA_LOCATION_PROPERTY + 
                    " ) appropriately." );
        }

        return dataLocation;
    }

    // LOG
    private static boolean deleteFile( String path, String fileName )
    throws SecurityException, RuntimeException {

        boolean rtnVal = false;
        File file = new File( path, fileName );

        if ( debug.ON ) {
            debug.out( "file( " + file + " )" );
        }

        if ( ! file.exists() ) {

            if ( debug.ON ) {
                debug.out( "file.exists()( " + file.exists() + " )" );
            }

            rtnVal = false;
            // do nothing
        }
        else if ( file.isFile() ) {

            // may throw SecurityException
            try {

                rtnVal = file.delete();

                if ( debug.ON ) {
                    debug.out( "deleted( " + rtnVal + " )" );
                }
            }
            catch ( RuntimeException ex ) {

                // LOG
                debug.out( "Unable to delete file( " + file.getAbsolutePath() +
                        "), exception( " + ex + " )", ex );
            }
        }
        // not a file
        else {

            // LOG
            debug.out( "File( " + file.getAbsolutePath() +
                    ") is not a file" );
            throw new RuntimeException( "File( " + file.getAbsolutePath() +
                    " ) is not a file" );
        }

        if ( rtnVal == false ) {

            // LOG
            debug.out( "Unable to delete file( " + file.getAbsolutePath() +
                    "), for unknown reason" );
        }

        return rtnVal;
    }

    // LOG
    // returns true if the file exists and is a file, not a directory
    private static boolean doesFileExist( String path, String fileName ) {

        boolean rtnVal = false;
        File file = new File( path, fileName );

        if ( debug.ON ) {
            debug.out( "file( " + file + " )" );
        }

        if ( ! file.exists() ) {

            if ( debug.ON ) {
                debug.out( "file.exists()( " + file.exists() + " )" );
            }

            rtnVal = false;
        }
        else if ( file.isFile() ) {

            rtnVal = true;
        }
        // not a file
        else {

            rtnVal = false;
        }

        return rtnVal;
    }

    private static String getProperty( String property )
    throws RuntimeException {

        String rtnVal;

        try {
            Properties properties = System.getProperties();

            rtnVal = (String) properties.get( property );
        }
        // possibly a SecurityException or a ClassCastException
        catch ( RuntimeException ex ) {

            // change to log message
            debug.out( "failed getting property( " + property +
                    " ), exception( " + ex + " )", ex );

            throw ex;
        }

        return rtnVal;
    }
}


