
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/QuoteParser.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.e2e.common;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.util.DefaultEntityResolver;
import com.bea.commerce.ebusiness.price.quote.Money;
import com.bea.commerce.ebusiness.price.quote.ImmutableMoney;

import weblogic.apache.xerces.parsers.SAXParser;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;

import java.io.InputStream;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;


public class QuoteParser implements ContentHandler, ErrorHandler {

    public static final String EMPTY_STRING = "";

    private static final SimpleDateFormat xmlDateFormat =
            new SimpleDateFormat( BuyerFactory.xmlDateFormatString );

    private static Debug debug =
            Debug.getInstance( QuoteParser.class );

    private QuoteElement element;
    private InputSource source;
    private Locator locator;
    private boolean errorFlag = false;
    private List quotes;
    private String cValue;
    private int productCnt = 0;

    private final Map fields =
            new HashMap( (int)( QuoteElement.SIZE / 0.75f ) + 1 );


    // set the system property to turn off XML validation
    private static final String VALIDATION_PROP =
            "samples.e2e.b2b.xml.validate";
    private static boolean validate = false;

    static {

        try {
            validate = Boolean.getBoolean( VALIDATION_PROP );

            if ( debug.ON ) {
                debug.out( "validate(" + validate + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "validate(" + validate + ")", e );
        }
    }


    public QuoteParser( InputSource source ) {

        this.source = source;
    } 


    public List parse() throws ParsingException {

        if ( source == null ) {

            return null;
        }

        if ( debug.ON ) {
            debug.out( "validate( " + validate + " )" );
        }

        try {

            XMLReader xmlReader = new SAXParser();

            if ( validate ) {

                // configure for validation

                xmlReader.setEntityResolver(
                        new DefaultEntityResolver( "/examples/e2e/common/dtd" ) );

                xmlReader.setFeature(
                        "http://xml.org/sax/features/validation", true );
            }

            xmlReader.setContentHandler( this );
            xmlReader.setErrorHandler( this );

            xmlReader.parse( source );

            InputStream is = source.getByteStream();

            if ( errorFlag ) {

                if ( debug.ON ) {
                    debug.out( "errorFlag(" + errorFlag + ")" );
                }

                throw new ParsingException( "parsing error encountered, see log" );
            }

        }
        catch ( SAXParseException ex ) {

            throw (ParsingException) ex.getException();
        }
        catch ( SAXException ex ) {

            throw new ParsingException( "SAXException(" + ex.getMessage() +
                    ") encountered processing quote XML", ex );
        }
        catch ( IOException ex ) {

            throw new ParsingException( "IOException(" + ex.getMessage() +
                    ") encountered processing quote XML", ex );
        }

        return list();
    }


    private void start( QuoteElement element ) {

        switch ( element.getInt() ) {

        // start of quote
        case QuoteElement.I_RESPONSE: {

                // clear old values
                clear();

                break;
            }                 

            // we recognize only a single product for a quote
            // ignore additional products
        case QuoteElement.I_AVAILABILITY: {

                productCnt++;

                if ( debug.ON ) {
                    debug.out( "productCnt(" + productCnt + ")" );
                }

                break;
            }
        }
    }

    private void end( QuoteElement element ) throws ParsingException {

        // more than 1 product for a quote not supported
        if ( productCnt > 1 ) {
            return;
        }

        switch ( element.getInt() ) {

        // end of quote
        case QuoteElement.I_RESPONSE: {

                // create object
                // add to list
                list().add( getQuote() );

                break;
            }

            // once per document
        case QuoteElement.I_REQUEST_ID:
            // once per quote
        case QuoteElement.I_RESPONSE_ID:
        case QuoteElement.I_SUPPLIER_NAME:
        case QuoteElement.I_PART_ID:
        case QuoteElement.I_PART_DESC:
        case QuoteElement.I_UNIT_CURRENCY:
        case QuoteElement.I_LOCATION:
        case QuoteElement.I_NOTE: {

                fields.put( element, getString() );
                break;
            }

            // once per document
        case QuoteElement.I_CREATION_DATE: 
            // once per quote
        case QuoteElement.I_RESPONSE_DATE:
        case QuoteElement.I_AVAILABLE_DATE: {

                fields.put( element, getCalendar() );
                break;
            }

        case QuoteElement.I_QUANTITY: {

                fields.put( element, getLong() );
                break;
            }

        case QuoteElement.I_UNIT_PRICE: {

                fields.put( element, getDouble() );
                break;
            }
        }
    }


    private String getString() {

        return cValue.trim();
    }

    private Calendar getCalendar() throws ParsingException {

        Date date = null;

        try {

            date = xmlDateFormat.parse( cValue );
        }
        catch ( java.text.ParseException ex ) {

            throw new ParsingException( "failed parsing Date for element( " +
                    element + " ) with value( " +
                    cValue + " ), message( " +
                    ex.getMessage() + " )", ex );
        }

        Calendar calendar = (Calendar) xmlDateFormat.getCalendar().clone();
        calendar.setTime( date );

        return calendar;
    }

    private Double getDouble() throws ParsingException {

        try {

            return new Double( cValue );
        }
        catch ( NumberFormatException ex ) {

            throw new ParsingException( "failed parsing Double for element( " +
                    element + " ) with value( " +
                    cValue + " ), message( " +
                    ex.getMessage() + " )", ex );
        }
    }

    private Long getLong() throws ParsingException {

        try {

            return new Long( cValue );
        }
        catch ( NumberFormatException ex ) {

            throw new ParsingException( "failed parsing Long for element( " +
                    element + " ) with value( " +
                    cValue + " ), message( " +
                    ex.getMessage() + " )", ex );
        }
    }

    private PurchaseQuote getQuote() throws ParsingException {

        PurchaseQuoteImpl quote = new PurchaseQuoteImpl();

        // temp variables
        String string = null;
        long longVal = 0;
        double doubleVal = 0.0d;
        Calendar calendar = null;

        string = (String) fields.get( QuoteElement.REQUEST_ID );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.REQUEST_ID +
                    "( " + string + " ) ]" );
        }
        quote.setRequestId( string );

        string = (String) fields.get( QuoteElement.RESPONSE_ID );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.RESPONSE_ID +
                    "( " + string + " ) ]" );
        }
        quote.setQuoteId( string );

        calendar = (Calendar) fields.get( QuoteElement.CREATION_DATE );
        if ( calendar == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.CREATION_DATE +
                    "( " + calendar + " ) ]" );
        }
        quote.setQpaCreationDate( calendar );

        calendar = (Calendar) fields.get( QuoteElement.RESPONSE_DATE );
        if ( calendar == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.RESPONSE_DATE +
                    "( " + calendar + " ) ]" );
        }
        quote.setQuoteDate( calendar );

        string = (String) fields.get( QuoteElement.SUPPLIER_NAME );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.SUPPLIER_NAME +
                    "( " + string + " ) ]" );
        }
        quote.setSupplierName( string );

        string = (String) fields.get( QuoteElement.PART_ID );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.PART_ID +
                    "( " + string + " ) ]" );
        }
        quote.setProductId( string );

        string = (String) fields.get( QuoteElement.PART_DESC );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.PART_DESC +
                    "( " + string + " ) ]" );
        }
        quote.setDescription( string );

        longVal = ((Long) fields.get( QuoteElement.QUANTITY )).longValue();
        if ( longVal < 0.0 ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.QUANTITY +
                    "( " + longVal + " ) ]" );
        }
        quote.setQuotedQty( longVal );

        // money
        doubleVal = ((Double) fields.get( QuoteElement.UNIT_PRICE )).doubleValue();
        if ( doubleVal < 0.0d ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.UNIT_PRICE +
                    "( " + doubleVal + " ) ]" );
        }

        string = (String) fields.get( QuoteElement.UNIT_CURRENCY );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.UNIT_CURRENCY +
                    "( " + string + " ) ]" );
        }
        quote.setQuotedUnitPrice( (Money) new ImmutableMoney( doubleVal, string ) );

        calendar = (Calendar) fields.get( QuoteElement.AVAILABLE_DATE );
        if ( calendar == null ) {

            throw new ParsingException(
                    "failed creating quote[ " +
                    QuoteElement.AVAILABLE_DATE +
                    "( " + calendar + " ) ]" );
        }
        quote.setPromiseDate( calendar );

        return quote;
    }

    // clear values out of the hash prior to parsing a new quote.
    private void clear() {

        String requestId = (String) fields.get( QuoteElement.REQUEST_ID );
        Calendar createDate = (Calendar) fields.get( QuoteElement.CREATION_DATE );

        fields.clear();

        // reset product cnt for the quote
        productCnt = 0;

        fields.put( QuoteElement.REQUEST_ID, requestId );
        fields.put( QuoteElement.CREATION_DATE, createDate );
    }

    private List list() {

        if ( quotes == null ) {

            quotes = new ArrayList( 2 );
        }

        return quotes;
    }


    //////////////////////////////////////////////
    // IMPLEMENTATION OF CONTENT HANDLER INTERFACE
    //////////////////////////////////////////////

    public void startElement(String namespaceURI,
            String localName,
            String qName,
            Attributes attributes) throws SAXException {

        if ( debug.ON ) {
            debug.out( "localName(" + localName + ")" );
            debug.out( "namespaceURI(" + namespaceURI + ")" );
            debug.out( "qName(" + qName + ")" );
        }

        element = QuoteElement.getObject( localName );

        if ( debug.ON ) {
            debug.out( "element(" + element + ")" );
        }

        if ( element == null ) {

            if ( debug.ON ) {
                debug.out( "unrecognized localName(" + localName + ")" );
            }
        }
        else {

            start( element );
        }
    }

    public void characters( char[] chars, int start, int length )
    throws SAXException {

        cValue = new String( chars, start, length );

        if ( debug.ON ) {
            debug.out( "cValue(" + cValue + ")" );
        }

    }

    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "localName(" + localName + ")" );
            debug.out( "namespaceURI(" + namespaceURI + ")" );
            debug.out( "qName(" + qName + ")" );
        }

        element = QuoteElement.getObject( localName );

        if ( debug.ON ) {
            debug.out( "element(" + element + ")" );
        }

        if ( element == null ) {

            if ( debug.ON ) {
                debug.out( "unrecognized localName(" + localName + ")" );
            }
        }
        else {

            try {
                end( element );
            }
            catch ( ParsingException ex ) {

                if ( debug.ON ) {
                    debug.out( "ex(" + ex + ")" );
                }

                throw new SAXParseException( ex.getMessage(), locator, ex );
            }
        }
    }

    public void ignorableWhitespace(char[] chars, int start, int length)
    throws SAXException {

        String wValue = new String( chars, start, length );

        if ( debug.ON ) {
            debug.out( "wValue(" + wValue + ")" );
        }

    }

    public void processingInstruction( String target, String data )
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "target(" + target + ")" );
            debug.out( "data(" + data + ")" );
        }

    }

    public void setDocumentLocator( Locator locator ) {

        if ( debug.ON ) {
            debug.out( "locator(" + locator + ")" );
        }

        this.locator = locator;
    }

    public void skippedEntity(String name)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "name(" + name + ")" );
        }

    }

    public void startDocument()
    throws SAXException {

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
        }

    }

    public void endDocument()
    throws SAXException {

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
        }

    }

    public void startPrefixMapping(String prefix, String uri)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "prefix(" + prefix + ")" );
            debug.out( "uri(" + uri + ")" );
        }

    }

    public void endPrefixMapping(String prefix)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "prefix(" + prefix + ")" );
        }

    }


    ////////////////////////////////////////////
    // IMPLEMENTATION OF ERROR HANDLER INTERFACE
    ////////////////////////////////////////////

    public void error(SAXParseException e)
    throws SAXException {
        errorFlag = true;

        if ( debug.ON ) {

            debug.out( "PARSE ERROR[ " +
                    "Public ID( " + e.getPublicId() + " ), " +
                    "System ID( " + e.getSystemId() + " ), " +
                    "Line number( " + e.getLineNumber() + " ), " +
                    "Column number(" + e.getColumnNumber() + " ) ]",
                    e );

        }
    }

    public void fatalError(SAXParseException e)
    throws SAXException {

        errorFlag = true;

        if ( debug.ON ) {

            debug.out( "FATAL PARSE ERROR[ " +
                    "Public ID( " + e.getPublicId() + " ), " +
                    "System ID( " + e.getSystemId() + " ), " +
                    "Line number( " + e.getLineNumber() + " ), " +
                    "Column number(" + e.getColumnNumber() + " ) ]",
                    e );

        }
    }

    public void warning(SAXParseException e)
    throws SAXException {

        errorFlag = true;

        if ( debug.ON ) {

            debug.out( "PARSE WARNING[ " +
                    "Public ID( " + e.getPublicId() + " ), " +
                    "System ID( " + e.getSystemId() + " ), " +
                    "Line number( " + e.getLineNumber() + " ), " +
                    "Column number(" + e.getColumnNumber() + " ) ]",
                    e );

        }
    }


}
