package examples.wlcs.sampleapp.payment;

import java.io.*;
import java.util.*;

/**
 * A PSResponse is a wrapper around a collection of PSResponseElements.
 * It holds the collection along with a response code and response message from
 * the Payment Service.  Not all payment vendors will return all information.   Please see the
 * documentation for your payment service for more information.
 *
 * @author BEA Systems, Inc.
 */
public class PSResponse
{

        public static final String PS_OK = "OK";

	private String	responseCode = null;
	private String	responseMessage = null;
	private PSResponseElement[]	responseArray = null;


	/**
	 * The default constructor
	 */
	public PSResponse()
	{
            makeResponseOK();
	}

	/**
	 * The response code is payment service specific.  It is usually used to indicate the
	 * success or failure of the request.
	 *
	 * @param code the code to return
	 */
	public void setResponseCode( String code )
	{
		responseCode = code;
	}

	/**
	 * The response message is payment service specific.  It is usually used in conjunction
	 * with the response code to provide a textual description of the response code.
	 *
	 * @param message the message to return
	 */
	public void setResponseMessage( String message )
	{
		responseMessage = message;
	}

	/**
	 * Set the response data array
	 *
	 * @param array the response array to be set
	 */
	public void setResponseArray( PSResponseElement[] array )
	{
    	responseArray = array;
	}

	/**
	 * Return the response code
         *
	 * @return the response code from the payment service call
	 */
	public String getResponseCode()
	{
		return( responseCode );
	}

	/**
	 * Return the response message from the payment service
	 *
	 * @return the message
         */
	public String getResponseMessage()
	{
		return( responseMessage );
	}

	/**
	 * The results are stored in a collection (in this case an Array) and can
	 * be retrieved from there.  Note that, depending on the payment service, this may
	 * return null if there where no results returned.  This can happen if an
	 * error occured.
	 *
	 * @return the results stored in an Array
         *
	 */
	public PSResponseElement[] getResponseArray()
	{
		return( responseArray );
	}

        /**
         * Convenience method for indicating that the payment service ecxecuted successfully/
         */        
        public void makeResponseOK()
        {
                responseCode = PS_OK;
        }

        /**
         * Convenience method for checking if the payment service ecxecuted successfully/
         */
        public boolean checkResponseOK()
        {
                return responseCode.equalsIgnoreCase(PS_OK);
        }

        /**
	 * The toString() method is used to generate debugging output
	 *
	 * @return the debugging java.lang.String to be displayed on the output
	 * of your choice.
	 */
	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String			lineSeparator = System.getProperty( "line.separator" );


		outputBuffer.append( "PSResponse contents:" + lineSeparator );
		outputBuffer.append( "\tresponseCode: \"" + responseCode + "\"" + lineSeparator );
		outputBuffer.append( "\tresponseMessage: \"" + responseMessage + "\"" + lineSeparator );

		if( responseArray != null )
		{
			outputBuffer.append( "\tnumber of elements returned: \"" + responseArray.length  + "\"" + lineSeparator );

			for( int i = 0; i < responseArray.length; i++ )
			{
				outputBuffer.append( "PSResponseElement #" + i + lineSeparator +
								  	responseArray[i].toString() );
			}
		}
		else
		{
			outputBuffer.append( "\tnumber of element returned: \"0\"" + lineSeparator );
		}

		return( outputBuffer.toString() );
	}
}
